use itertools::Itertools;

fn main() {
    let input = aoc::input!();
    println!("part 1: {}", part1(&input));
    println!("part 1: {}", part2(&input));
}

fn part1(input: &str) -> i64 {
    input
        .split("\n\n") // double newline means new elf inventory
        .map(|items| items.lines().map(|item| item.parse::<i64>().unwrap()).sum())
        .max()
        .unwrap()
}

fn part2(input: &str) -> i64 {
    input
        .split("\n\n") // double newline means new elf inventory
        .map(|items| {
            items
                .lines()
                .map(|item| item.parse::<i64>().unwrap())
                .sum::<i64>()
        })
        .sorted_unstable()
        .rev()
        .take(3)
        .sum::<i64>()
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn examples() {
        let example = "1000\n2000\n3000\n\n4000\n\n5000\n6000\n\n7000\n8000\n9000\n\n10000";
        assert_eq!(part1(example), 24000);
        assert_eq!(part2(example), 45000);
    }
}
